<?php
/**
 * Jeg Portfolio Class
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package jeg-paywall
 */

namespace Jeg\Portfolio;

use Jeg\Portfolio\Metaboxes\Metaboxes;
use Jeg\Portfolio\Dashboard\Dashboard;
use Jeg\Portfolio\Customizer\Customizer;
use Jeg\Portfolio\Template\Template;
use Jeg\Portfolio\Asset\Asset;
use Jeg\Portfolio\Ajax\Ajax;


/**
 * Class Init
 *
 * @package Jeg\Portfolio
 */
class Init {

	/**
	 * @var Init
	 */
	private static $instance;

	/**
	 * Listener Endpoint
	 *
	 * @var $endpoint
	 */
	private $endpoint = 'jeg-portfolio';

	/**
	 * Init constructor.
	 */
	private function __construct() {
		$this->setup_init();
		$this->setup_hook();
	}

	/**
	 * @return Init
	 */
	public static function instance() {
		if ( null === static::$instance ) {
			static::$instance = new static();
		}

		return static::$instance;
	}

	/**
	 * Setup Classes
	 */
	private function setup_init() {
		Metaboxes::instance();
		Dashboard::instance();
		Customizer::instance();
		Template::instance();
		Asset::instance();
		Ajax::instance();
	}

	/**
	 * Setup Hooks
	 */
	private function setup_hook() {
		add_action( 'init', [ $this, 'register_post_type' ] );
		add_action( 'init', [ $this, 'register_taxonomy' ] );
		add_filter( 'manage_edit-portfolio_columns', [ $this, 'register_custom_column' ] );
		add_action( 'manage_posts_custom_column', [ $this, 'register_column_field' ] );
		add_action( 'get_header', [ $this, 'jeg_init_page_variable' ] );
		add_action( 'save_post', [ $this, 'update_portfolio_parent' ], 99, 3 );
		add_action( 'save_post', [ $this, 'update_attachment_url' ], 99, 3 );
		add_action( 'portfolio_share', [ $this, 'portfolio_share' ] );
		add_action( 'portfolio_post_share', [ $this, 'post_share' ], null, 2 );
		add_filter( 'pre_get_posts', [ $this, 'searchfilter' ] );
		add_filter( 'preview_post_link', [ $this, 'change_preview_link' ], null, 2 );

		// styles
		add_action( 'admin_enqueue_scripts', [ $this, 'editor_style' ] );
		add_action( 'wp_enqueue_scripts', [ $this, 'load_frontend_css' ] );

		// scripts
		add_action( 'admin_enqueue_scripts', [ $this, 'register_portfolio_scripts' ] );

		// fonts
		add_filter( 'jeg_fonts_option_setting', [ $this, 'register_fonts_option' ], 99 );
	}

	/**
	 * Load editor style
	 */
	public function editor_style() {
		wp_enqueue_style( 'jeg-portfolio-admin', JEG_PORTFOLIO_URL . '/assets/css/admin/admin-style.css', null, JEG_PORTFOLIO_VERSION );
	}

	/**
	 * Register Portfolio Scripts
	 */
	function register_portfolio_scripts() {
		$screen = get_current_screen();

		if ( 'portfolio' === $screen->post_type && is_admin() ) {
			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'jeg-portfolio-metabox', JEG_PORTFOLIO_URL . '/assets/js/admin/portfolio-metabox.js', null, JEG_PORTFOLIO_VERSION );
		}

		if ( 'page' === $screen->post_type && is_admin() ) {
			wp_enqueue_script( 'jeg-portfolio-admin', JEG_PORTFOLIO_URL . '/assets/js/admin/pagemetabox.js', null, JEG_PORTFOLIO_VERSION );
		}
	}

	/**
	 * Load Frontend CSS
	 */
	public function load_frontend_css() {
		if ( ! jeg_portfolio_is_login_page() && is_jeg_portfolio_page() ) {
			if ( ! is_admin() && ! SCRIPT_DEBUG ) {
				wp_enqueue_style( 'jeg-portfolio-frontend', JEG_PORTFOLIO_URL . '/assets/css/frontend/style.min.css', null, JEG_PORTFOLIO_VERSION );
				wp_enqueue_style( 'fontawesome', JEG_PORTFOLIO_URL . '/assets/css/frontend/fontawesome/font-awesome.min.css', null, JEG_PORTFOLIO_VERSION );
			} else {
				wp_enqueue_style( 'fontawesome', JEG_PORTFOLIO_URL . '/assets/css/frontend/fontawesome/font-awesome.min.css', null, JEG_PORTFOLIO_VERSION );
				wp_enqueue_style( 'jeg-portfolio-frontend', JEG_PORTFOLIO_URL . '/assets/css/frontend/frontend-style.css', null, JEG_PORTFOLIO_VERSION );
				wp_enqueue_style( 'jeg-portfolio-media', JEG_PORTFOLIO_URL . '/assets/css/frontend/media-style.css', null, JEG_PORTFOLIO_VERSION );
				wp_enqueue_style( 'jeg-portfolio-responsive', JEG_PORTFOLIO_URL . '/assets/css/frontend/responsive-style.css', null, JEG_PORTFOLIO_VERSION );
			}
		}

	}

	/**
	 * register page id
	 */
	public function jeg_init_page_variable() {
		global $post;
		if ( $post !== null ) {
			if ( ! is_404() ) {
				defined( 'JEG_PAGE_ID' ) or define( 'JEG_PAGE_ID', $post->ID );
				jeg_portfolio_set_post_views( $post->ID );
			}
		} else {
			defined( 'JEG_PAGE_ID' ) or define( 'JEG_PAGE_ID', 0 );
		}

	}

	/**
	 * Register Portfolio Post Type
	 */
	public function register_post_type() {
		$args =
			[
				'labels'          =>
					[
						'name'               => 'Portfolio',
						'singular_name'      => 'Portfolios',
						'add_new'            => 'Add Portfolio',
						'add_new_item'       => 'Add Portfolio',
						'edit_item'          => 'Edit Portfolio',
						'new_item'           => 'New Portfolio',
						'view_item'          => 'View Item',
						'search_items'       => 'Search Portfolio Items',
						'not_found'          => 'No portfolio items found',
						'not_found_in_trash' => 'No portfolio items found in Trash',
						'parent_item_colon'  => '',
					],
				'description'     => 'Portfolio Post type',
				'public'          => true,
				'show_ui'         => true,
				'menu_icon'       => JEG_PORTFOLIO_URL . '/assets/img/portfolio.png',
				'menu_position'   => 6,
				'capability_type' => 'post',
				'hierarchical'    => false,
				'supports'        => [ 'title', 'editor', 'comments', 'page-attributes' ],
				'taxonomies'      => [ 'portfolio_category' ],
			];
		$args['exclude_from_search'] = jeg_portfolio_get_option( 'search_exclude_portfolio', false );
		register_post_type( 'portfolio', $args );
	}

	/**
	 * Register Portfolio Taxonomy
	 */
	public function register_taxonomy() {
		register_taxonomy(
			'portfolio_category',
			[ 'portfolio' ],
			[
				'hierarchical'   => true,
				'label'          => 'Portfolio Categories',
				'singular_label' => 'Portfolio Categories',
				'rewrite'        => true,
				'query_var'      => true,
			]
		);
	}

	/**
	 * Register Portfolio Custom Column
	 *
	 * @param $columns
	 *
	 * @return array
	 */
	function register_custom_column( $columns ) {

		if ( isset( $columns['date'] ) ) {
			unset( $columns['date'] );
		}

		$columns = array_merge(
			$columns,
			[
				'portfoliotemplate' => 'Portfolio Template',
				'thumbwidth'        => 'Thumb Width',
				'thumbheight'       => 'Thumb Height',
				'portfolioparent'   => 'Portfolio Parent',
			]
		);

		return $columns;
	}

	/***
	 * additional information on post page
	 *
	 * @param $name
	 */
	function register_column_field( $name ) {
		global $post;
		global $typenow;

		switch ( $name ) {
			case 'portfoliotemplate':
				$layout = jeg_metabox( 'jpt_setting_metabox.portfolio_layout', 'ajax_portfolio', $post->ID );
				switch ( $layout ) {
					case 'ajax_portfolio':
						echo 'Ajax Portfolio';
						break;
					case 'cover':
						echo 'Cover portfolio';
						break;
					case 'side_content_portfolio':
						echo 'Side content';
						break;
					case 'landingpage':
						echo 'Landing page';
						break;
					case 'landingpagevc':
						echo 'Landing page';
						break;
					case 'to_another_page_link':
						echo 'Link to another page';
						break;
					default:
						break;
				}
				break;
			case 'thumbwidth':
				$coversize = jeg_metabox( 'jpt_setting_metabox.thumb_width', '1', $post->ID );
				switch ( $coversize ) {
					case '0.25':
						echo '1/4x width';
						break;
					case '0.5':
						echo '1/2x width';
						break;
					case '1':
						echo '1x width';
						break;
					case '2':
						echo '2x width';
						break;
					case '3':
						echo '3x width';
						break;
					default:
						break;
				}
				break;
			case 'thumbheight':
				$coversize = jeg_metabox( 'jpt_setting_metabox.thumb_height', '1', $post->ID );
				switch ( $coversize ) {
					case '0.25':
						echo '1/4x height';
						break;
					case '0.5':
						echo '1/2x height';
						break;
					case '1':
						echo '1x height';
						break;
					case '2':
						echo '2x height';
						break;
					case '3':
						echo '3x height';
						break;
					default:
						break;
				}
				break;
			case 'portfolioparent':
				$id       = jeg_metabox( 'jpt_setting_metabox.portfolio_parent', 'ajax_portfolio', $post->ID );
				$parent   = get_post( $id );
				$editlink = admin_url( 'edit.php' ) . '?post_type=portfolio&portfolio_parent=' . $id;
				echo "<a href='" . $editlink . "'>" . $parent->post_title . '</a>';
				break;
		}
	}

	/**
	 * Update portfolio parent to post meta
	 *
	 * @param $post_ID
	 * @param $post
	 * @param $update
	 */
	public function update_portfolio_parent( $post_ID, $post, $update ) {
		if ( 'portfolio' === $post->post_type ) {
			$parent_id = jeg_metabox( 'jpt_setting_metabox.portfolio_parent', null, $post->ID );
			update_post_meta( $post->ID, 'portfolio_parent', $parent_id );
		}
	}

	/**
	 * Update attachment URL
	 *
	 * @param $post_ID
	 * @param $post
	 * @param $update
	 */
	public function update_attachment_url( $post_ID, $post, $update ) {
		$meta_box_id = 'jpt_media_builder_metabox';
		if ( 'portfolio' === $post->post_type ) {
			if ( isset( $_POST[ $meta_box_id ] ) && wp_verify_nonce( sanitize_key( $_POST[ $meta_box_id ]['nonce'] ), $meta_box_id ) ) {
				$data = get_post_meta( $post_ID, $meta_box_id, true );

				foreach ( $data['portfolio_media_builder'] as $key => $media ) {
					if ( is_numeric( $media['media_image'] ) ) {
						$data['portfolio_media_builder'][ $key ]['media_image']    = wp_get_attachment_url( intval( $media['media_image'] ) );
						$data['portfolio_media_builder'][ $key ]['media_image_id'] = $media['media_image'];
					}
					if ( is_numeric( $media['image_cover'] ) ) {
						$data['portfolio_media_builder'][ $key ]['image_cover']    = wp_get_attachment_url( intval( $media['image_cover'] ) );
						$data['portfolio_media_builder'][ $key ]['image_cover_id'] = $media['image_cover'];
					}
					if ( is_numeric( $media['video_mp4'] ) ) {
						$data['portfolio_media_builder'][ $key ]['video_mp4'] = wp_get_attachment_url( intval( $media['video_mp4'] ) );
					}
					if ( is_numeric( $media['video_webm'] ) ) {
						$data['portfolio_media_builder'][ $key ]['video_webm'] = wp_get_attachment_url( intval( $media['video_webm'] ) );
					}
					if ( is_numeric( $media['video_ogg'] ) ) {
						$data['portfolio_media_builder'][ $key ]['video_ogg'] = wp_get_attachment_url( intval( $media['video_ogg'] ) );
					}
				}

				update_post_meta( $post_ID, $meta_box_id, $data );
			}
		}
	}

	/**
	 * Portfolio Share Button
	 */
	public function portfolio_share() {
		?>
		<div class="share-body">
			<?php
			$share_socials = jeg_portfolio_get_option( 'share_socials', [ 'facebook', 'twitter' ] );
			foreach ( $share_socials as $social ) {
				switch ( $social ) {
					case 'facebook':
						?>
						<div data-id="facebook" class="share-facebook">
							<div class="share-text"><?php esc_html_e( 'Share on Facebook', 'jeg-portfolio' ); ?></div>
						</div>
						<?php
						break;
					case 'twitter':
						?>
						<div data-id="twitter" class="share-twitter">
							<div class="share-text"><?php esc_html_e( 'Share on Twitter', 'jeg-portfolio' ); ?></div>
						</div>
						<?php
						break;
					case 'pinterest':
						?>
						<div data-id="pinterest" class="share-pinterest">
							<div class="share-text"><?php esc_html_e( 'Share on Pinterest', 'jeg-portfolio' ); ?></div>
						</div>
						<?php
						break;
					case 'linkedin':
						?>
						<div data-id="linkedin" class="share-linkedin">
							<div class="share-text"><?php esc_html_e( 'Share on LinkedIn', 'jeg-portfolio' ); ?></div>
						</div>
						<?php
						break;
					case 'vk':
						?>
						<div data-id="vk" class="share-vk">
							<div class="share-text"><?php esc_html_e( 'Share on VK', 'jeg-portfolio' ); ?></div>
						</div>
						<?php
						break;
					case 'email':
						?>
						<div data-id="email" class="share-email">
							<div class="share-text"><?php esc_html_e( 'Share on Email', 'jeg-portfolio' ); ?></div>
						</div>
						<?php
						break;
				}
			}
			?>
		</div>
		<?php
	}

	public function post_share( $posttitle, $featured_img ) {
		?>
		<div class="sharing-icons">
			<?php
			$share_socials = jeg_portfolio_get_option( 'share_socials', [ 'facebook', 'twitter' ] );
			foreach ( $share_socials as $social ) {
				switch ( $social ) {
					case 'facebook':
						?>
						<div class="sharing-icon sharing-facebook socials-share">
							<a data-shareto="Facebook" target="_blank"
							href="https://www.facebook.com/sharer/sharer.php?u=<?php echo wp_get_shortlink(); ?>"><i
									class="fa fa-facebook"></i></a>
						</div>
						<?php
						break;
					case 'twitter':
						?>
						<div class="sharing-icon sharing-twitter socials-share">
							<a data-shareto="Twitter" target="_blank"
							   href="http://twitter.com/intent/tweet?url=<?php echo wp_get_shortlink(); ?>&text=<?php echo jeg_sanitize_output( $posttitle ); ?>"><i
									class="fa fa-twitter"></i></a>
						</div>
						<?php
						break;
					case 'pinterest':
						?>
						<div class="sharing-icon sharing-pinterest socials-share">
							<a data-shareto="Pinterest" target="_blank"
							href="http://pinterest.com/pin/create/button/?url=<?php echo wp_get_shortlink(); ?>&media=<?php echo jeg_sanitize_output( $featured_img ); ?>&decription=<?php echo jeg_sanitize_output( $posttitle ); ?>"><i
									class="fa fa-pinterest"></i></a>
						</div>
						<?php
						break;
					case 'linkedin':
						?>
						<div class="sharing-icon sharing-linkedin socials-share">
							<a data-shareto="LinkedIn" target="_blank"
							   href="https://www.linkedin.com/shareArticle?url=<?php echo wp_get_shortlink(); ?>&title=<?php echo jeg_sanitize_output( $posttitle ); ?>"><i
									class="fa fa-linkedin"></i></a>
						</div>
						<?php
						break;
					case 'vk':
						?>
						<div class="sharing-icon sharing-vk socials-share">
							<a data-shareto="VK" target="_blank"
							href="http://vk.com/share.php?url=<?php echo wp_get_shortlink(); ?>"><i class="fa fa-vk"></i></a>
						</div>
						<?php
						break;
					case 'email':
						?>
						<div class="sharing-icon sharing-email socials-share">
							<a data-shareto="Email" target="_blank"
							   href="mailto:?subject=<?php echo jeg_sanitize_output( $posttitle ); ?>&amp;body=<?php echo wp_get_shortlink(); ?>"><i
									class="fa fa-envelope"></i></a>
						</div>
						<?php
						break;
				}
			}
			?>
		</div>
		<?php
	}

	/**
	 * @param $query
	 *
	 * @return mixed
	 */
	public function searchfilter( $query ) {
		if ( $query->is_search && ! is_admin() ) {
			if ( isset( $_GET['post_type'] ) ) {
				if ( 'portfolio' === $_GET['post_type'] ) {
					$query->set( 'post_type', [ 'portfolio' ] );
				}
			}
		}
		return $query;
	}

	/**
	 * @param $fonts
	 *
	 * @return array
	 */
	public function register_fonts_option( $fonts ) {
		$fonts = array_merge(
			$fonts,
			[
				'jeg_portfolio_option[title_font]',
				'jeg_portfolio_option[paragraph_font]',
				'jeg_portfolio_option[meta_title_font]',
				'jeg_portfolio_option[meta_description_font]',
				'jeg_portfolio_option[nav_font]',
				'jeg_portfolio_option[nav_bottom_font]',
				'jeg_portfolio_option[nav_popup_font]',
				'jeg_portfolio_option[slider_button_font]',
				'jeg_portfolio_option[share_title_font]',
				'jeg_portfolio_option[share_container_font]',
				'jeg_portfolio_option[date_font]',
				'jeg_portfolio_option[thumbnail_category_font]',
				'jeg_portfolio_option[thumbnail_title_font]',
				'jeg_portfolio_option[filter_font]',
				'jeg_portfolio_option[filter_list_font]',
				'jeg_portfolio_option[search_font]',
			]
		);

		return $fonts;
	}

	public function change_preview_link( $link, $post ) {
		if ( isset( $_POST['post_type'] ) && 'portfolio' === $_POST['post_type'] ) {
			if ( isset( $_POST['jpt_setting_metabox'] ) && isset( $_POST['jpt_setting_metabox']['portfolio_layout'] ) && 'to_another_page_link' === $_POST['jpt_setting_metabox']['portfolio_layout'] ) {
				if ( isset( $_POST['jpt_another_page_setting'] ) && isset( $_POST['jpt_another_page_setting']['page_link'] ) ) {
					return $_POST['jpt_another_page_setting']['page_link'];
				}
			}
		}

		return $link;
	}

}
