<?php

/**
 * JEG Portfolio Class
 *
 * @package jeg-portfolio
 *
 * @author Jegtheme
 *
 * @since 1.0.0
 */

namespace Jeg\Portfolio\Template;

/**
 * Class Template
 *
 * @package Jeg\Portfolio\Template
 */
class Template {
    /**
     * @var Template
     */
    private static $instance;

    /**
     * The array of templates that this plugin tracks.
     */
    protected $templates;

    /**
     * Metaboxes constructor.
     */
    public function __construct() {
        $this->setup_init();
        $this->setup_hook();
    }

    /**
     * @return Template
     */
    public static function instance() {
        if ( null === static::$instance ) {
            static::$instance = new static();
        }

        return static::$instance;
    }

    /**
     * Setup Classes
     */
    private function setup_init() {
    }

    /**
     * Setup Hooks
     */
    private function setup_hook() {
        $this->templates = [];

        if ( version_compare( floatval( get_bloginfo( 'version' ) ), '4.7', '<' ) ) {
            // 4.6 and older
            add_filter(
                'page_attributes_dropdown_pages_args',
                [$this, 'register_project_templates']
            );
        } else {
            // Add a filter to the wp 4.7 version attributes metabox
            add_filter(
                'theme_page_templates',
                [$this, 'add_new_template']
            );
        }

        add_filter(
            'wp_insert_post_data',
            [$this, 'register_project_templates']
        );

        add_filter(
            'template_include',
            [$this, 'view_project_template']
        );

        add_filter(
            'template_include',
            [$this, 'view_single_post_template']
        );

        add_filter(
            'template_include',
            [$this, 'view_search_template']
        );

        $this->templates = [
            'template-portfolio.php' => esc_html__( 'Portfolio List', 'jeg_portfolio' ),
        ];
    }

    /**
     * Adds template to the page dropdown for v4.7+
     *
     * @param  $posts_templates
     * @return array
     */
    public function add_new_template( $posts_templates ) {
        $posts_templates = array_merge( $posts_templates, $this->templates );

        return $posts_templates;
    }

    /**
     * Adds template to the pages cache in order to trick WordPress
     *
     * @param  $atts
     * @return mixed
     */
    public function register_project_templates( $atts ) {
        $cache_key = 'page_templates-' . md5( get_theme_root() . '/' . get_stylesheet() );

        $templates = wp_get_theme()->get_page_templates();

        if ( empty( $templates ) ) {
            $templates = [];
        }

        wp_cache_delete( $cache_key, 'themes' );

        $templates = array_merge( $templates, $this->templates );

        wp_cache_add( $cache_key, $templates, 'themes', 1800 );

        return $atts;
    }

    /**
     * Checks if the template is assigned to the page
     *
     * @param  $template
     * @return string
     */
    public function view_project_template( $template ) {
        if ( is_search() ) {
            return $template;
        }

        global $post;

        if ( ! $post ) {
            return $template;
        }

        if ( ! isset(
            $this->templates[get_post_meta(
                $post->ID,
                '_wp_page_template',
                true
            )]
        ) ) {
            return $template;
        }

        $file = plugin_dir_path( __FILE__ ) . 'templates/';

        $filepath = apply_filters( 'page_templater_plugin_dir_path', $file );

        $file = $filepath . get_post_meta(
            $post->ID,
            '_wp_page_template',
            true
        );

        if ( file_exists( $file ) ) {
            return $file;
        } else {
            echo jeg_sanitize_output( $file );
        }

        return $template;
    }

    /**
     * Checks template for single posts
     *
     * @param  $template
     * @return string
     */
    public function view_single_post_template( $template ) {
        global $post;

        if ( ! $post ) {
            return $template;
        }

        if ( ! isset( $post->post_type ) || ( 'portfolio' !== $post->post_type ) ) {
            return $template;
        }

        if ( ! is_single() ) {
            return $template;
        }

        $file = plugin_dir_path( __FILE__ ) . 'templates/template-single-post.php';

        if ( ! file_exists( $file ) ) {
            return $template;
        }

        add_filter( 'body_class', [$this, 'add_body_class'] );

        return $file;
    }

    /**
     * Checks template for search
     *
     * @param  $template
     * @return string
     */
    public function view_search_template( $template ) {
        if ( ! is_search() ) {
            return $template;
        }

        if ( ! isset( $_GET['post_type'] ) || ( 'portfolio' !== $_GET['post_type'] ) ) {
            return $template;
        }

        $file = plugin_dir_path( __FILE__ ) . 'templates/template-search.php';

        if ( ! file_exists( $file ) ) {
            return $template;
        }

        return $file;
    }

    /**
     * Add jeg-portfolio-page to body class
     *
     * @param  $classes
     * @return array
     */
    public function add_body_class( $classes ) {
        return array_merge( $classes, ['jeg-portfolio-page'] );
    }
}
